var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { vi, test, describe, beforeEach, afterEach, expect, } from "vitest";
import { withSentry, getNetlifyConfig, startSentryCheckIn, completeSentryCheckIn,
// Ignore this rule for the time being.
// See https://github.com/microsoft/TypeScript/issues/49083
// This was still an issue as of TS v4.9, hopefully gets sorted as 
// the current 'fix' is to add a '.js' extension which doesn't make sense
// since that file doesn't actually exist.
// @ts-ignore TS2835
 } from "../src/index";
import Chance from "chance";
vi.mock("@sentry/serverless", (importOriginal) => __awaiter(void 0, void 0, void 0, function* () {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const mod = yield importOriginal();
    const wrapHandler = mod.AWSLambda.wrapHandler;
    return Object.assign(Object.assign({}, mod), { AWSLambda: {
            init: vi.fn(),
            wrapHandler,
        }, configureScope: vi.fn() });
}));
vi.mock("node-fetch", (importOriginal) => __awaiter(void 0, void 0, void 0, function* () {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const mod = yield importOriginal();
    return Object.assign(Object.assign({}, mod), { default: vi.fn() });
}));
vi.mock('../src/ampli/index', () => {
    const mocks = {
        isLoaded: true,
        load: vi.fn().mockReturnValue({ promise: Promise.resolve() }),
        flush: vi.fn().mockReturnValue({ promise: Promise.resolve() }),
        integrationSentryEnabled: vi.fn(),
        integrationSentryCronMonitoringEnabled: vi.fn()
    };
    return {
        ampli: mocks
    };
});
const chance = new Chance();
/**
 * This is the same as what's in 'getCronMonitoringCheckInRequestHeaders' but duplicated here
 * in order to:
 *
 * 1) Make it easy for us to construct request headers for mock requests
 * 2) Ensure that if someone accidentally modifies the request headers in the other function
 * that it's caught in these tests
 */
const constructSentryRequestHeaders = () => {
    return {
        'Authorization': `DSN ${process.env.SENTRY_DSN}`,
        'Content-Type': "application/json"
    };
};
describe("getNetlifyConfig", () => {
    let handlerConfiguration;
    beforeEach(() => {
        handlerConfiguration = {
            sentry: {
                tracesSampleRate: chance.floating({ fixed: 1, min: 0, max: 1 }),
                cronMonitoring: {
                    enable: true,
                    monitorId: chance.guid(),
                },
            },
        };
    });
    test("does not modify Sentry configuration", () => {
        var _a, _b;
        const originalValue = (_a = handlerConfiguration.sentry) === null || _a === void 0 ? void 0 : _a.tracesSampleRate;
        getNetlifyConfig(handlerConfiguration.sentry);
        expect((_b = handlerConfiguration.sentry) === null || _b === void 0 ? void 0 : _b.tracesSampleRate).toEqual(originalValue);
    });
    test("removes Netlify Sentry configuration from the user-provided configuration object", () => {
        var _a;
        getNetlifyConfig(handlerConfiguration.sentry);
        expect((_a = handlerConfiguration.sentry) === null || _a === void 0 ? void 0 : _a.cronMonitoring).not.toBeDefined();
    });
    test("returns the Netlify Sentry configuration", () => {
        var _a;
        const originalCronMonitoringConfig = {
            cronMonitoring: (_a = handlerConfiguration.sentry) === null || _a === void 0 ? void 0 : _a.cronMonitoring,
        };
        const netlifySentryConfig = getNetlifyConfig(handlerConfiguration.sentry);
        expect(netlifySentryConfig).toStrictEqual(originalCronMonitoringConfig);
    });
});
describe("startSentryCheckIn", () => {
    beforeEach(() => {
        process.env.SENTRY_DSN = `https://${chance.string({
            alpha: true,
            numeric: true,
        })}@${chance.string({
            alpha: true,
            numeric: true,
        })}.ingest.sentry.io/${chance.integer({ min: 0 })}`;
    });
    afterEach(() => {
        vi.resetAllMocks();
        delete process.env.SENTRY_DSN;
    });
    test("makes the request with the correct headers", () => __awaiter(void 0, void 0, void 0, function* () {
        const fetchMock = (yield import("node-fetch"))
            .default;
        const monitorId = chance.guid();
        yield startSentryCheckIn(monitorId);
        const expectedUrl = `https://sentry.io/api/0/monitors/${monitorId}/checkins/`;
        const expectedPayload = {
            method: "POST",
            headers: constructSentryRequestHeaders(),
            body: JSON.stringify({ status: "in_progress" }),
        };
        expect(fetchMock).toHaveBeenCalledWith(expectedUrl, expectedPayload);
    }));
});
describe("completeSentryCheckIn", () => {
    beforeEach(() => {
        process.env.SENTRY_DSN = `https://${chance.string({
            alpha: true,
            numeric: true,
        })}@${chance.string({
            alpha: true,
            numeric: true,
        })}.ingest.sentry.io/${chance.integer({ min: 0 })}`;
    });
    afterEach(() => {
        vi.resetAllMocks();
        delete process.env.SENTRY_DSN;
    });
    test("makes the request with the correct headers", () => __awaiter(void 0, void 0, void 0, function* () {
        const fetchMock = (yield import("node-fetch"))
            .default;
        const monitorId = chance.guid();
        yield completeSentryCheckIn(monitorId);
        const expectedUrl = `https://sentry.io/api/0/monitors/${monitorId}/checkins/latest/`;
        const expectedPayload = {
            method: "PUT",
            headers: constructSentryRequestHeaders(),
            body: JSON.stringify({ status: "ok" }),
        };
        expect(fetchMock).toHaveBeenCalledWith(expectedUrl, expectedPayload);
    }));
});
describe("withSentry", () => {
    let fetchMock;
    let mockHandler;
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        fetchMock = (yield import("node-fetch")).default;
        process.env.SENTRY_DSN = `https://${chance.string({
            alpha: true,
            numeric: true,
        })}@${chance.string({
            alpha: true,
            numeric: true,
        })}.ingest.sentry.io/${chance.integer({ min: 0 })}`;
        mockHandler = (
        /* eslint-disable @typescript-eslint/no-unused-vars */
        event, context
        /* eslint-enable @typescript-eslint/no-unused-vars */
        ) => __awaiter(void 0, void 0, void 0, function* () {
            return {
                statusCode: 200,
            };
        });
    }));
    afterEach(() => {
        vi.resetAllMocks();
        delete process.env.SENTRY_DSN;
    });
    test("it performs Sentry check ins when cron monitoring is enabled", () => __awaiter(void 0, void 0, void 0, function* () {
        const monitorId = chance.guid();
        const mockConfig = {
            sentry: {
                cronMonitoring: {
                    enable: true,
                    monitorId,
                },
            },
        };
        const res = withSentry(mockHandler, mockConfig);
        yield res({}, {});
        expect(fetchMock).toHaveBeenCalledTimes(2);
        expect(fetchMock.mock.calls[0][0]).toEqual(`https://sentry.io/api/0/monitors/${monitorId}/checkins/`);
        expect(fetchMock.mock.calls[1][0]).toEqual(`https://sentry.io/api/0/monitors/${monitorId}/checkins/latest/`);
    }));
    test("it does not perform Sentry check ins when cron monitoring is disabled", () => __awaiter(void 0, void 0, void 0, function* () {
        const monitorId = chance.guid();
        const mockConfig = {
            sentry: {
                cronMonitoring: {
                    enable: false,
                    monitorId,
                },
            },
        };
        const res = withSentry(mockHandler, mockConfig);
        yield res({}, {});
        expect(fetchMock).not.toBeCalled();
    }));
    test("it calls configureScope in order to set the transaction name to the filename of the function", () => __awaiter(void 0, void 0, void 0, function* () {
        const configureScopeMock = (yield import("@sentry/serverless"))
            .configureScope;
        const monitorId = chance.guid();
        const mockConfig = {
            sentry: {
                cronMonitoring: {
                    enable: false,
                    monitorId,
                },
            },
        };
        const res = withSentry(mockHandler, mockConfig);
        yield res({}, {});
        expect(configureScopeMock).toBeCalled();
    }));
});
